package com.diagbot.facade;

import com.alibaba.fastjson.JSONObject;
import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.ResponseData;
import com.diagbot.biz.push.vo.SearchVO;
import com.diagbot.biz.push.entity.Treat;
import com.diagbot.biz.push.enums.FeatureTypeEnum;
import com.diagbot.dto.PushDTO;
import com.diagbot.enums.SysTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.StringUtil;
import io.github.lvyahui8.spring.aggregate.facade.DataBeanAggregateQueryFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/5/7 16:12
 */
@Component
public class PushFacade {
    @Autowired
    private ClinicalFacade clinicalFacade;
    @Autowired
    private TreatmentFacade treatmentFacade;
    @Autowired
    private TransferFacade transferFacade;
    @Autowired
    DataBeanAggregateQueryFacade dataBeanAggregateQueryFacade;

    /**
     * 推理接口
     *
     * @param searchVO
     * @return ResponseData
     */
    public PushDTO pushInner(SearchVO searchVO) {
        PushDTO pushDTO = new PushDTO();
        //是否对接
        Boolean isConnect = transferFacade.isConnect(searchVO.getHosCode(), searchVO.getSysType());
        //入参预处理
        searchVO = precSearchData(searchVO, isConnect);
        ResponseData data = clinicalFacade.processClinicalData(searchVO);

        //是否有病情提示标志
        pushDTO.setHasIndications(data.getHasIndications());

        String featureType = searchVO.getFeatureType();
        String[] featureTypes = featureType.split(",|，");
        Set<String> featureTypeSet = new HashSet(Arrays.asList(featureTypes));

        try {
            Map<String, Object> invokeParams = new HashMap<>();
            invokeParams.put("pushDTO",pushDTO);
            invokeParams.put("featureTypeSet", featureTypeSet);
            invokeParams.put("data", data);
            invokeParams.put("isConnect", isConnect);
            invokeParams.put("hosCode", searchVO.getHosCode());
            pushDTO
                    = dataBeanAggregateQueryFacade.get("setAll", invokeParams, PushDTO.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
        }

        //治疗方案单独处理
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Feature_Type_Treat.getKey()))) {
            pushDTO.setTreat(getTreatment(data.getTreat(), searchVO.getDiseaseName(), searchVO.getDisType()));
        }
        return pushDTO;
    }

    /**
     * 获取治疗方案
     *
     * @param
     * @return
     */
    public Treat getTreatment(Treat treat, Item diseaseName, Integer disType) {
        if (StringUtil.isBlank(diseaseName.getUniqueName())) {
            throw new CommonException(CommonErrorCode.PARAM_ERROR, "请输入需获取治疗方案的诊断名称");
        }
        treat = treatmentFacade.getTreatment(treat, diseaseName, disType);
        return treat;
    }

    /**
     * 获取管理评估动态推送内容（来源大数据）
     *
     * @param searchVO
     * @return
     */
    public Map<String, JSONObject> getManagementEvaluationContent(SearchVO searchVO) {
        ResponseData data = clinicalFacade.processClinicalData(searchVO);

        String featureType = searchVO.getFeatureType();
        String[] featureTypes = featureType.split(",|，");
        Set<String> featureTypeSet = new HashSet(Arrays.asList(featureTypes));
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Feature_Type_ManagementEvaluation.getKey()))) {
            return data.getManagementEvaluation();
        } else {
            return null;
        }
    }

    /**
     * searchData入参预处理
     *
     * @param searchVO
     * @param isConnect
     * @return
     */
    public SearchVO precSearchData(SearchVO searchVO, Boolean isConnect) {
        //未对接或非数据引擎模式，不做映射
        if (!isConnect || (!searchVO.getSysType().equals(SysTypeEnum.LTAPI_SERVICE.getKey()))) {
            return searchVO;
        }
        try {
            Map<String, Object> inputParams = new HashMap<>();
            inputParams.put("searchVO", searchVO);
            inputParams.put("isConnect", isConnect);
            searchVO
                    = dataBeanAggregateQueryFacade.get("assembleSearchData", inputParams, SearchVO.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
        }
        return searchVO;
    }
}